from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.on_fail_options import OnFailOptions

T = TypeVar("T", bound="OnFail")


@_attrs_define
class OnFail:
    """
    Attributes:
        validator_tag (str):
        method (OnFailOptions):
    """

    validator_tag: str
    method: OnFailOptions
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        validator_tag = self.validator_tag

        method = self.method.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "validatorTag": validator_tag,
                "method": method,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        validator_tag = d.pop("validatorTag")

        method = OnFailOptions(d.pop("method"))

        on_fail = cls(
            validator_tag=validator_tag,
            method=method,
        )

        on_fail.additional_properties = d
        return on_fail

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
